module division
    (input wire reset,
     input wire clk,
     input wire [15:0] thigh,
     input wire [15:0] tlow,
     input wire start,
     output reg [31:0] QUO,
     output reg [31:0] ACC,
     output reg out_enable
	);

reg [31:0] A;
reg [31:0] B;
reg [63:0] ACCT; 
reg [16:0] sum;
reg [7:0] status; 
reg [7:0] count;
	
always @(negedge clk)     // for a detailed discussion of the working principle of the division algorithm
begin                     // see the web site https://projectf.io/posts/division-in-verilog/
   if (reset == 1)        // thigh (16-bit) stores the measurement of tH
   begin                  // tlow (16-bit) stores the measurement of tL
      status <= 0;        // QUO (32-bit) stores the division result
      A <= 0;
      B <= 0;
      ACCT <= 0;
      ACC <= 0;
      QUO <= 0; 
      count <= 0;
      out_enable <= 0;
   end
   else
   begin            
      case (status) 
         0: begin           
               if (start == 1)
               begin              
                  A <= {thigh, 16'd0};
                  sum <= thigh + tlow;    
                  ACC <= 0;
                  QUO <= 0; 
                  count <= 0; 
                  out_enable <= 0;
                  status <= 1;
               end
               else
                  status <= 0;
            end 
         1: begin          
               B <= {15'd0, sum}; 
               ACCT <= {ACC, A};
               status <= 2;
            end 
         2: begin 
               ACCT <= ACCT << 1;
               QUO <= QUO << 1;
               status <= 3;
            end
         3: begin           
               ACC <= ACCT[63:32];
               A <= ACCT[31:0];
               status <= 4;
            end 
         4: begin             
               if (ACC >= B)
               begin     
                  ACC <= ACC - B;
                  QUO <= {QUO[15:1], 1'b1};   
               end
               status <= 5;
            end 
         5: begin 
                ACCT <= {ACC, A};  
               if (count == 31) 
               begin 
                  out_enable <= 1;
                  status <= 6;
               end
               else
               begin
                  count <= count + 1;
                  status <= 2;
               end 
            end
         6: begin  
               out_enable <= 0;
               status <= 0;
            end
      endcase
   end
end 
		
endmodule