module UART_control (
                     input reset,
                     input wire clk, 
                     input wire [7:0] UART_RX_data,
                     input wire UART_RX_valid,
                     input wire UART_TX_completed, 
                     input wire [31:0] data_result,
                     input wire data_acquired, 
                     output reg [7:0] UART_TX_data,
                     output reg UART_TX_valid
                     );

reg [3:0] status_machine;
reg [7:0] number_of_data;
reg [7:0] byte3;
reg [7:0] byte2;
reg [7:0] byte1;
reg [7:0] byte0;

always @ (posedge clk)
begin                 
   if (reset == 1)
   begin 
      status_machine <= 0; 
      number_of_data <= 0;        
      byte3 <= 0;
      byte2 <= 0;
      byte1 <= 0;
      byte0 <= 0;
      UART_TX_valid <= 0;
      UART_TX_data <= 0;  
   end 
   else
   begin      
      case (status_machine)   
         0: begin                         
               if (UART_RX_valid == 1)
               begin    
                  case (UART_RX_data)                        // when decimal code 10 is received, a measurement cycle is started
                     7'd10: status_machine <= 1;
                     default: status_machine <= 0;
                  endcase
               end 
               else
                  status_machine <= 0;  
            end
         1: begin           
               status_machine <= 2;      
            end                    
         2: begin  
               if (data_acquired == 1)        // the system waits data_acquired = 1
               begin                          // that means the tH and tL measurements were carried out
                  status_machine <= 3;        // and the division operation was completed
               end                            // the 32-bit register data_result stores the division result
               else                           // the 32-bit data_result are transferred to the PC using UART with 4 8-bit data transfers
                  status_machine <= 2;
            end   
         3: begin  
               byte3 <= data_result[31:24];
               byte2 <= data_result[23:16];
               byte1 <= data_result[15:8];
               byte0 <= data_result[7:0];
               status_machine <= 4;
            end
         4: begin  
               UART_TX_data <= byte3;                  // First 8-bit data transfer using UART
               UART_TX_valid <= 1;
               status_machine <= 5;
            end                    
         5: begin                                      // when the signal UART_TX_completed is set
               UART_TX_valid <= 0;                     // it indicates that the UART data transfer was completed
               if (UART_TX_completed == 1)
                  status_machine <= 6;
               else
                  status_machine <= 5;
            end
         6: begin                                     // Second 8-bit data transfer using UART
               UART_TX_data <= byte2;
               UART_TX_valid <= 1;
               status_machine <= 7;
            end                    
         7: begin
               UART_TX_valid <= 0; 
               if (UART_TX_completed == 1)
                  status_machine <= 8;
               else
                  status_machine <= 7;
            end
         8: begin                                    // Third 8-bit data transfer using UART
               UART_TX_data <= byte1;
               UART_TX_valid <= 1;
               status_machine <= 9;
            end                    
         9: begin
               UART_TX_valid <= 0; 
               if (UART_TX_completed == 1)
                  status_machine <= 10;
               else
                  status_machine <= 9;
            end
         10:begin                                    // Fourth 8-bit data transfer using UART
               UART_TX_data <= byte0;
               UART_TX_valid <= 1;
               status_machine <= 11;
            end                    
         11:begin
               UART_TX_valid <= 0; 
               if (UART_TX_completed == 1)
                  status_machine <= 0;
               else
                  status_machine <= 11;
            end
         default: status_machine <= 0;
      endcase
   end  
end

endmodule