module count_periods
    (input wire reset,
     input wire clk,
     input wire x1,
     output reg x2,
     output reg [15:0] thigh,
     output reg [15:0] tlow,
     output reg start
	);

reg [15:0] count;
reg [7:0] status_count;
reg x1_t;
reg x1_d;

always @ (negedge clk)         // initialization of the 2-stages flip-flip register for clock-domain-crossing
begin
   if (reset == 1)
   begin
      x1_t <= 0;
      x1_d <= 0;
   end
   else
   begin
      x1_t <= ~x1;
      x1_d <= x1_t;
   end
end

always @ (posedge clk)
begin 
   if (reset == 1)
   begin    
      thigh <= 0;
      tlow <= 0;  
      start <= 0;
      count <= 0;
      status_count <= 0;
   end
   else
   begin     
      case (status_count)    
         0: begin               // wait for a stable state of signal x1_d
               start <= 0;
               if (x1_d == 1)  
                  status_count <= 1; 
               else
                  status_count <= 0;
            end
         1: begin                  // x1_d = 1  
               if (x1_d == 0)     // wait for the transition x1_d: 1 -> 0
               begin  
                  count <= 1;
                  status_count <= 2;
               end 
               else
                  status_count <= 1;   
            end
         2: begin                // measurement of time tL   
               start <= 0;
               if (x1_d == 1)     // transition to status_count 3 when x1_d: 0 -> 1
               begin  
                  tlow <= count;
                  count <= 1;
                  status_count <= 3;
               end 
               else
               begin 
                  count <= count + 1;
                  status_count <= 2;  
               end 
            end
         3: begin                // measurement of time tH
               if (x1_d == 0)     // transition to status_count 0 when x1_d: 1 -> 0
               begin  
                  thigh <= count;
                  count <= 1;
                  start <= 1;
                  status_count <= 0;
               end 
               else
               begin 
                  count <= count + 1;
                  status_count <= 3;  
               end 
            end
      endcase
   end
end
  
always @ (posedge clk)          // generation of the signal output v2
begin 
   if (reset == 1)
   begin    
      x2 <= 0;
   end
   else
   begin 
      if (x1_d == 0)
         x2 <= 0;
      else
         x2 <= 1;
   end     
end
  
endmodule